@@ -18,6 +18,31 @@
  * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
+/* 
+ *  [ ctwm ]
+ *
+ *  Copyright 1992 Claude Lecommandeur.
+ *            
+ * Permission to use, copy, modify  and distribute this software  [ctwm] and
+ * its documentation for any purpose is hereby granted without fee, provided
+ * that the above  copyright notice appear  in all copies and that both that
+ * copyright notice and this permission notice appear in supporting documen-
+ * tation, and that the name of  Claude Lecommandeur not be used in adverti-
+ * sing or  publicity  pertaining to  distribution of  the software  without
+ * specific, written prior permission. Claude Lecommandeur make no represen-
+ * tations  about the suitability  of this software  for any purpose.  It is
+ * provided "as is" without express or implied warranty.
+ *
+ * Claude Lecommandeur DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+ * INCLUDING ALL  IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS.  IN NO
+ * EVENT SHALL  Claude Lecommandeur  BE LIABLE FOR ANY SPECIAL,  INDIRECT OR
+ * CONSEQUENTIAL  DAMAGES OR ANY  DAMAGES WHATSOEVER  RESULTING FROM LOSS OF
+ * USE, DATA  OR PROFITS,  WHETHER IN AN ACTION  OF CONTRACT,  NEGLIGENCE OR
+ * OTHER  TORTIOUS ACTION,  ARISING OUT OF OR IN  CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
+ *
+ * Author:  Claude Lecommandeur [ lecom@sic.epfl.ch ][ April 1992 ]
+ */
 
 /***********************************************************************
  *
@@ -43,14 +68,20 @@
 #include "resize.h"
 #include "add_window.h"
 #include "siconify.bm"
+#ifdef VMS
+#include <decw$include/Xos.h>
+#include <X11Xmu/CharSet.h>
+#else
 #include <X11/Xos.h>
 #include <X11/Xmu/CharSet.h>
+#endif
 #ifdef macII
 int strcmp(); /* missing from string.h in AUX 2.0 */
 #endif
 
 int iconmgr_textx = siconify_width+11;
 WList *Active = NULL;
+WList *Current = NULL;
 WList *DownIconManager = NULL;
 int iconifybox_width = siconify_width;
 int iconifybox_height = siconify_height;
@@ -79,10 +110,13 @@ void CreateIconManagers()
     Pixel background;
     char *icon_name;
     WorkSpaceList *wlist;
+    XWMHints	  wmhints;
+    int bw;
 
     if (Scr->NoIconManagers)
 	return;
 
+    if (Scr->use3Diconmanagers) iconmgr_textx += Scr->IconManagerShadowDepth;
     if (Scr->siconifyPm == None)
     {
 	Scr->siconifyPm = XCreatePixmapFromBitmapData(dpy, Scr->Root,
@@ -93,25 +127,6 @@ void CreateIconManagers()
     for (q = Scr->iconmgr; q != NULL; q = q->nextv) {
       for (p = q; p != NULL; p = p->next)
       {
-	mask = XParseGeometry(p->geometry, &JunkX, &JunkY,
-			      (unsigned int *) &p->width, (unsigned int *)&p->height);
-
-	if (mask & XNegative)
-	    JunkX = Scr->MyDisplayWidth - p->width - 
-	      (2 * Scr->BorderWidth) + JunkX;
-
-	if (mask & YNegative)
-	    JunkY = Scr->MyDisplayHeight - p->height - 
-	      (2 * Scr->BorderWidth) + JunkY;
-
-	background = Scr->IconManagerC.back;
-	GetColorFromList(Scr->IconManagerBL, p->name, (XClassHint *)NULL,
-			 &background);
-
-	p->w = XCreateSimpleWindow(dpy, Scr->Root,
-	    JunkX, JunkY, p->width, p->height, 1,
-	    Scr->Black, background);
-
 	sprintf(str, "%s Icon Manager", p->name);
 	sprintf(str1, "%s Icons", p->name);
 	if (p->icon_name)
@@ -119,11 +134,50 @@ void CreateIconManagers()
 	else
 	    icon_name = str1;
 
+	if (!p->geometry || !strlen(p->geometry)) p->geometry = "+0+0";
+	mask = XParseGeometry(p->geometry, &JunkX, &JunkY,
+			      (unsigned int *) &p->width, (unsigned int *)&p->height);
+
+	bw = LookInList (Scr->NoBorder, str, NULL) ? 0 :
+		(Scr->ThreeDBorderWidth ? Scr->ThreeDBorderWidth : Scr->BorderWidth);
+
+	if (mask & XNegative)
+	    JunkX += Scr->MyDisplayWidth - p->width - 2 * bw;
+
+	if (mask & YNegative)
+	    JunkY += Scr->MyDisplayHeight - p->height - 2 * bw;
+
+	background = Scr->IconManagerC.back;
+	GetColorFromList(Scr->IconManagerBL, p->name, (XClassHint *)NULL,
+			 &background);
+
+	if (p->width  < 1) p->width  = 1;
+	if (p->height < 1) p->height = 1;
+	p->w = XCreateSimpleWindow(dpy, Scr->Root,
+	    JunkX, JunkY, p->width, p->height, 1,
+	    Scr->Black, background);
+
 	XSetStandardProperties(dpy, p->w, str, icon_name, None, NULL, 0, NULL);
 
 	Scr->workSpaceMgr.activeWSPC = wlist;
+	wmhints.initial_state = NormalState;
+	wmhints.input         = True;
+	XSetWMHints (dpy, p->w, &wmhints);
 	p->twm_win = AddWindow(p->w, TRUE, p);
+
+	p->twm_win->mapped = FALSE;
 	SetMapStateProp (p->twm_win, WithdrawnState);
+	if (p->twm_win && p->twm_win->wmhints &&
+	    (p->twm_win->wmhints->initial_state == IconicState)) {
+	    p->twm_win->isicon = TRUE;
+	}
+	else
+	if (!Scr->NoIconManagers && Scr->ShowIconManager) {
+	    p->twm_win->isicon = FALSE;
+	}
+	else {
+	    p->twm_win->isicon = TRUE;
+	}
       }
       if (wlist != NULL) wlist = wlist->next;
     }
@@ -171,6 +225,7 @@ IconMgr *AllocateIconManager(name, icon_name, geom, columns)
     if (Scr->NoIconManagers)
 	return NULL;
 
+    if (columns < 1) columns = 1;
     p = (IconMgr *)malloc(sizeof(IconMgr));
     p->name = name;
     p->icon_name = icon_name;
@@ -230,11 +285,11 @@ void MoveIconManager(dir)
     int row_inc, col_inc;
     int got_it;
 
-    if (!Active) return;
+    if (!Current) return;
 
-    cur_row = Active->row;
-    cur_col = Active->col;
-    ip = Active->iconmgr;
+    cur_row = Current->row;
+    cur_col = Current->col;
+    ip = Current->iconmgr;
 
     row_inc = 0;
     col_inc = 0;
@@ -243,13 +298,13 @@ void MoveIconManager(dir)
     switch (dir)
     {
 	case F_FORWICONMGR:
-	    if ((tmp = Active->next) == NULL)
+	    if ((tmp = Current->next) == NULL)
 		tmp = ip->first;
 	    got_it = TRUE;
 	    break;
 
 	case F_BACKICONMGR:
-	    if ((tmp = Active->prev) == NULL)
+	    if ((tmp = Current->prev) == NULL)
 		tmp = ip->last;
 	    got_it = TRUE;
 	    break;
@@ -317,12 +372,12 @@ void MoveIconManager(dir)
 
     /* raise the frame so the icon manager is visible */
     if (ip->twm_win->mapped) {
-	XRaiseWindow(dpy, ip->twm_win->frame);
+	RaiseWindow(ip->twm_win);
 	XWarpPointer(dpy, None, tmp->icon, 0,0,0,0, 5, 5);
     } else {
 	if (tmp->twm->title_height) {
 	    int tbx = Scr->TBInfo.titlex;
-	    int x = tmp->twm->highlightx;
+	    int x = tmp->twm->highlightxr;
 	    XWarpPointer (dpy, None, tmp->twm->title_w, 0, 0, 0, 0,
 			  tbx + (x - tbx) / 2,
 			  Scr->TitleHeight / 4);
@@ -354,7 +409,7 @@ void JumpIconManager(dir)
     ScreenInfo *sp;
     int screen;
 
-    if (!Active) return;
+    if (!Current) return;
 
 
 #define ITER(i) (dir == F_NEXTICONMGR ? (i)->next : (i)->prev)
@@ -362,7 +417,7 @@ void JumpIconManager(dir)
 #define TEST(ip) if ((ip)->count != 0 && (ip)->twm_win->mapped) \
 		 { got_it = TRUE; break; }
 
-    ip = Active->iconmgr;
+    ip = Current->iconmgr;
     for (tmp_ip = ITER(ip); tmp_ip; tmp_ip = ITER(tmp_ip)) {
 	TEST (tmp_ip);
     }
@@ -397,7 +452,7 @@ void JumpIconManager(dir)
     }
 
     /* raise the frame so it is visible */
-    XRaiseWindow(dpy, tmp_ip->twm_win->frame);
+    RaiseWindow(tmp_ip->twm_win);
     if (tmp_ip->active)
 	XWarpPointer(dpy, None, tmp_ip->active->icon, 0,0,0,0, 5, 5);
     else
@@ -419,7 +474,7 @@ WList *AddIconManager(tmp_win)
     TwmWindow *tmp_win;
 {
     WList *tmp, *old;
-    int h;
+    int h, offs;
     unsigned long valuemask;		/* mask for create windows */
     XSetWindowAttributes attributes;	/* attributes for create windows */
     IconMgr *ip;
@@ -469,8 +524,10 @@ WList *AddIconManager(tmp_win)
     if (Scr->use3Diconmanagers) {
 	if (!Scr->BeNiceToColormap) GetShadeColors (&tmp->cp);
 	tmp->iconifypm = Create3DIconManagerIcon (tmp->cp);
+	h = Scr->IconManagerFont.height + 2 * Scr->IconManagerShadowDepth + 6;
     }
-    h = Scr->IconManagerFont.height + 10;
+    else
+	h = Scr->IconManagerFont.height + 10;
     if (h < (siconify_height + 4))
 	h = siconify_height + 4;
 
@@ -491,14 +548,15 @@ WList *AddIconManager(tmp_win)
 			    CopyFromParent, (unsigned int) CopyFromParent,
 			    (Visual *) CopyFromParent, valuemask, &attributes);
 
-
     valuemask = (CWBackPixel | CWBorderPixel | CWEventMask | CWCursor);
     attributes.background_pixel = tmp->cp.back;
     attributes.border_pixel = Scr->Black;
     attributes.event_mask = (ButtonReleaseMask| ButtonPressMask |
 			     ExposureMask);
     attributes.cursor = Scr->ButtonCursor;
-    tmp->icon = XCreateWindow (dpy, tmp->w, 5, (int) (h - siconify_height)/2,
+    offs = Scr->use3Diconmanagers ? Scr->IconManagerShadowDepth : 2;
+    tmp->icon = XCreateWindow (dpy, tmp->w, offs + 3,
+			       (int) (h - siconify_height)/2,
 			       (unsigned int) siconify_width,
 			       (unsigned int) siconify_height,
 			       (unsigned int) 0, CopyFromParent,
@@ -508,7 +566,7 @@ WList *AddIconManager(tmp_win)
 
     ip->count += 1;
     PackIconManager(ip);
-    if (windowmask) XRaiseWindow (dpy, windowmask);
+    if (Scr->WindowMask) XRaiseWindow (dpy, Scr->WindowMask);
     XMapWindow(dpy, tmp->w);
 
     XSaveContext(dpy, tmp->w, IconManagerContext, (caddr_t) tmp);
@@ -517,16 +575,12 @@ WList *AddIconManager(tmp_win)
     XSaveContext(dpy, tmp->icon, TwmContext, (caddr_t) tmp_win);
     XSaveContext(dpy, tmp->icon, ScreenContext, (caddr_t) Scr);
 
-    if (! Scr->ShowIconManager) {
-	ip->twm_win->mapped = FALSE;
-	ip->twm_win->isicon = TRUE;
-    }
     if (!ip->twm_win->isicon)
     {
       if (OCCUPY (ip->twm_win, Scr->workSpaceMgr.activeWSPC)) {
 	SetMapStateProp (ip->twm_win, NormalState);
-	XMapWindow(dpy, ip->w);
-	XMapWindow(dpy, ip->twm_win->frame);
+	XMapWindow (dpy, ip->w);
+	XMapWindow (dpy, ip->twm_win->frame);
       }
 	ip->twm_win->mapped = TRUE;
     }
@@ -534,7 +588,7 @@ WList *AddIconManager(tmp_win)
     old = tmp;
     ip = ip->nextv;
   }
-    if (tmp == NULL) return;
+    if (tmp == NULL) return NULL;
     tmp_win->list = tmp;
     if (! OCCUPY (tmp->iconmgr->twm_win, Scr->workSpaceMgr.activeWSPC)) {
 	old = tmp;
@@ -623,6 +677,7 @@ void RemoveFromIconManager(ip, tmp)
 	ip->last = tmp->prev;
     else
 	tmp->next->prev = tmp->prev;
+    if (Current == tmp) Current = NULL;
 }
 
 /***********************************************************************
@@ -682,12 +737,19 @@ void RemoveIconManager(tmp_win)
   }
 }
 
+void CurrentIconManagerEntry (current)
+WList *current;
+{
+    Current = current;
+}
+
 void ActiveIconManager(active)
     WList *active;
 {
     active->active = TRUE;
     Active = active;
     Active->iconmgr->active = active;
+    Current = Active;
     DrawIconManagerBorder(active, False);
 }
 
@@ -703,14 +765,11 @@ void DrawIconManagerBorder(tmp, fill)
     int fill;
 {
     if (Scr->use3Diconmanagers) {
-	int shadow_width;
-
-	shadow_width = 2;
 	if (tmp->active && Scr->Highlight)
-	    Draw3DBorder (tmp->w, 0, 0, tmp->width, tmp->height, shadow_width,
+	    Draw3DBorder (tmp->w, 0, 0, tmp->width, tmp->height, Scr->IconManagerShadowDepth,
 				tmp->cp, on, fill, False);
 	else
-	    Draw3DBorder (tmp->w, 0, 0, tmp->width, tmp->height, shadow_width,
+	    Draw3DBorder (tmp->w, 0, 0, tmp->width, tmp->height, Scr->IconManagerShadowDepth,
 				tmp->cp, off, fill, False);
     }
     else {
@@ -790,8 +849,14 @@ void PackIconManager(ip)
     int new_x, new_y;
     int savewidth;
     WList *tmp;
+    int mask, bw;
+    unsigned int JunkW, JunkH;
 
-    wheight = Scr->IconManagerFont.height + 10;
+    if (Scr->use3Diconmanagers) {
+	wheight = Scr->IconManagerFont.height + 2 * Scr->IconManagerShadowDepth + 6;
+    } else {
+	wheight = Scr->IconManagerFont.height + 10;
+    }
     if (wheight < (siconify_height + 4))
 	wheight = siconify_height + 4;
 
@@ -844,10 +909,20 @@ void PackIconManager(ip)
 
     XResizeWindow(dpy, ip->w, newwidth, ip->height);
 
+    mask = XParseGeometry (ip->geometry, &JunkX, &JunkY, &JunkW, &JunkH);
+    if (mask & XNegative) {
+	ip->twm_win->frame_x += ip->twm_win->frame_width - newwidth -
+				2 * ip->twm_win->frame_bw3D;
+    }
+    if (mask & YNegative) {
+	ip->twm_win->frame_y += ip->twm_win->frame_height - ip->height -
+				2 * ip->twm_win->frame_bw3D - ip->twm_win->title_height;
+    }
     savewidth = ip->width;
     if (ip->twm_win)
       SetupWindow (ip->twm_win,
 		   ip->twm_win->frame_x, ip->twm_win->frame_y,
-		   newwidth, ip->height + ip->twm_win->title_height, -1);
+		   newwidth + 2 * ip->twm_win->frame_bw3D,
+		   ip->height + ip->twm_win->title_height + 2 * ip->twm_win->frame_bw3D, -1);
     ip->width = savewidth;
 }
