@@ -24,6 +24,31 @@
 /**    TORTIOUS ACTION, ARISING OUT OF OR IN  CONNECTION  WITH  THE  USE    **/
 /**    OR PERFORMANCE OF THIS SOFTWARE.                                     **/
 /*****************************************************************************/
+/* 
+ *  [ ctwm ]
+ *
+ *  Copyright 1992 Claude Lecommandeur.
+ *            
+ * Permission to use, copy, modify  and distribute this software  [ctwm] and
+ * its documentation for any purpose is hereby granted without fee, provided
+ * that the above  copyright notice appear  in all copies and that both that
+ * copyright notice and this permission notice appear in supporting documen-
+ * tation, and that the name of  Claude Lecommandeur not be used in adverti-
+ * sing or  publicity  pertaining to  distribution of  the software  without
+ * specific, written prior permission. Claude Lecommandeur make no represen-
+ * tations  about the suitability  of this software  for any purpose.  It is
+ * provided "as is" without express or implied warranty.
+ *
+ * Claude Lecommandeur DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+ * INCLUDING ALL  IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS.  IN NO
+ * EVENT SHALL  Claude Lecommandeur  BE LIABLE FOR ANY SPECIAL,  INDIRECT OR
+ * CONSEQUENTIAL  DAMAGES OR ANY  DAMAGES WHATSOEVER  RESULTING FROM LOSS OF
+ * USE, DATA  OR PROFITS,  WHETHER IN AN ACTION  OF CONTRACT,  NEGLIGENCE OR
+ * OTHER  TORTIOUS ACTION,  ARISING OUT OF OR IN  CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
+ *
+ * Author:  Claude Lecommandeur [ lecom@sic.epfl.ch ][ April 1992 ]
+ */
 
 
 /***********************************************************************
@@ -73,6 +98,11 @@ static int clampDY;
 static int last_width;
 static int last_height;
 
+static unsigned int resizeGrabMask;
+
+extern Cursor	TopRightCursor, TopLeftCursor, BottomRightCursor, BottomLeftCursor,
+		LeftCursor, RightCursor, TopCursor, BottomCursor;
+
 static void do_auto_clamp (tmp_win, evp)
     TwmWindow *tmp_win;
     XEvent *evp;
@@ -132,21 +162,23 @@ static void do_auto_clamp (tmp_win, evp)
  */
 
 void
-StartResize(evp, tmp_win, fromtitlebar)
+StartResize(evp, tmp_win, fromtitlebar, from3dborder)
 XEvent *evp;
 TwmWindow *tmp_win;
-Bool fromtitlebar;
+Bool fromtitlebar, from3dborder;
 {
     Window      junkRoot;
     unsigned int junkbw, junkDepth;
+    Cursor	cursor;
 
+    cursor = (Scr->BorderCursors && tmp_win->curcurs) ? tmp_win->curcurs : Scr->ResizeCursor;
     ResizeWindow = tmp_win->frame;
     if (! Scr->OpaqueResize || resizeWhenAdd) XGrabServer(dpy);
-    XGrabPointer(dpy, Scr->Root, True,
-        ButtonPressMask | ButtonReleaseMask |
-	ButtonMotionMask | PointerMotionHintMask,
+    resizeGrabMask = ButtonPressMask | ButtonReleaseMask |
+			ButtonMotionMask | PointerMotionHintMask;
+    XGrabPointer(dpy, Scr->Root, True, resizeGrabMask,
         GrabModeAsync, GrabModeAsync,
-        Scr->Root, Scr->ResizeCursor, CurrentTime);
+        Scr->Root, cursor, CurrentTime);
 
     XGetGeometry(dpy, (Drawable) tmp_win->frame, &junkRoot,
         &dragx, &dragy, (unsigned int *)&dragWidth, (unsigned int *)&dragHeight, &junkbw,
@@ -159,7 +191,7 @@ Bool fromtitlebar;
     origHeight = dragHeight;
     clampTop = clampBottom = clampLeft = clampRight = clampDX = clampDY = 0;
 
-    if (Scr->AutoRelativeResize && !fromtitlebar)
+    if (Scr->AutoRelativeResize && (from3dborder || !fromtitlebar))
       do_auto_clamp (tmp_win, evp);
 
     Scr->SizeStringOffset = SIZE_HINDENT;
@@ -176,7 +208,7 @@ Bool fromtitlebar;
         MoveOutline (Scr->Root, dragx - tmp_win->frame_bw,
 		 dragy - tmp_win->frame_bw, dragWidth + 2 * tmp_win->frame_bw,
 		 dragHeight + 2 * tmp_win->frame_bw,
-		 tmp_win->frame_bw, tmp_win->title_height);
+		 tmp_win->frame_bw, tmp_win->title_height + tmp_win->frame_bw3D);
 }
 
 
@@ -190,8 +222,8 @@ int x, y, w, h;
   unsigned int junkbw, junkDepth;
 
     if (! Scr->OpaqueResize) XGrabServer(dpy);
-    XGrabPointer(dpy, Scr->Root, True,
-        ButtonPressMask | ButtonMotionMask | PointerMotionMask,
+    resizeGrabMask = ButtonPressMask | ButtonMotionMask | PointerMotionMask;
+    XGrabPointer(dpy, Scr->Root, True, resizeGrabMask,
         GrabModeAsync, GrabModeAsync,
         Scr->Root, Scr->ResizeCursor, CurrentTime);
     dragx = x + tmp_win->frame_bw;
@@ -214,7 +246,7 @@ int x, y, w, h;
 		 dragy - tmp_win->frame_bw, 
 		 dragWidth + 2 * tmp_win->frame_bw,
 		 dragHeight + 2 * tmp_win->frame_bw,
-		 tmp_win->frame_bw, tmp_win->title_height);
+		 tmp_win->frame_bw, tmp_win->title_height + tmp_win->frame_bw3D);
 }
 
 /***********************************************************************
@@ -234,8 +266,8 @@ TwmWindow *tmp_win;
 int x, y, w, h;
 {
     XGrabServer(dpy);
-    XGrabPointer(dpy, Scr->Root, True,
-        ButtonReleaseMask | ButtonMotionMask | PointerMotionHintMask,
+    resizeGrabMask = ButtonReleaseMask | ButtonMotionMask | PointerMotionHintMask;
+    XGrabPointer(dpy, Scr->Root, True, resizeGrabMask,
         GrabModeAsync, GrabModeAsync,
         Scr->Root, Scr->ResizeCursor, CurrentTime);
 
@@ -265,6 +297,7 @@ int y_root;
 TwmWindow *tmp_win;
 {
     int action;
+    Cursor cursor;
 
     action = 0;
 
@@ -280,6 +313,7 @@ TwmWindow *tmp_win;
         dragy += delta;
         dragHeight -= delta;
         action = 1;
+	cursor = TopCursor;
     }
     else if (y_root <= dragy/* ||
              y_root == findRootInfo(root)->rooty*/) {
@@ -290,6 +324,7 @@ TwmWindow *tmp_win;
         clampTop = 1;
 	clampDY = 0;
         action = 1;
+	cursor = TopCursor;
     }
     if (clampLeft) {
         int         delta = x_root - dragx;
@@ -300,6 +335,7 @@ TwmWindow *tmp_win;
         dragx += delta;
         dragWidth -= delta;
         action = 1;
+	cursor = clampTop ? TopLeftCursor : LeftCursor;
     }
     else if (x_root <= dragx/* ||
              x_root == findRootInfo(root)->rootx*/) {
@@ -310,6 +346,7 @@ TwmWindow *tmp_win;
         clampLeft = 1;
 	clampDX = 0;
         action = 1;
+	cursor = clampTop ? TopLeftCursor : LeftCursor;
     }
     if (clampBottom) {
         int         delta = y_root - dragy - dragHeight;
@@ -319,6 +356,7 @@ TwmWindow *tmp_win;
         }
         dragHeight += delta;
         action = 1;
+	cursor = clampLeft ? BottomLeftCursor : BottomCursor;
     }
     else if (y_root >= dragy + dragHeight) {
         dragy = origy;
@@ -327,6 +365,7 @@ TwmWindow *tmp_win;
         clampBottom = 1;
 	clampDY = 0;
         action = 1;
+	cursor = clampLeft ? BottomLeftCursor : BottomCursor;
     }
     if (clampRight) {
         int         delta = x_root - dragx - dragWidth;
@@ -336,6 +375,8 @@ TwmWindow *tmp_win;
         }
         dragWidth += delta;
         action = 1;
+	cursor = clampBottom ? BottomRightCursor : RightCursor;
+	cursor = clampTop ? TopRightCursor : cursor;
     }
     else if (x_root >= dragx + dragWidth) {
         dragx = origx;
@@ -344,6 +385,8 @@ TwmWindow *tmp_win;
         clampRight = 1;
 	clampDX = 0;
         action = 1;
+	cursor = clampBottom ? BottomRightCursor : RightCursor;
+	cursor = clampTop ? TopRightCursor : cursor;
     }
 
     if (action) {
@@ -361,7 +404,11 @@ TwmWindow *tmp_win;
             dragy - tmp_win->frame_bw,
             dragWidth + 2 * tmp_win->frame_bw,
             dragHeight + 2 * tmp_win->frame_bw,
-	    tmp_win->frame_bw, tmp_win->title_height);
+	    tmp_win->frame_bw, tmp_win->title_height + tmp_win->frame_bw3D);
+	if (Scr->BorderCursors && (cursor != tmp_win->curcurs)) {
+	    tmp_win->curcurs = cursor;
+	    XChangeActivePointerGrab (dpy, resizeGrabMask, cursor, CurrentTime);
+	}
     }
 
     DisplaySize(tmp_win, dragWidth, dragHeight);
@@ -388,6 +435,7 @@ int y_root;
 TwmWindow *tmp_win;
 {
     int action;
+    Cursor cursor;
 
     action = 0;
 
@@ -403,6 +451,7 @@ TwmWindow *tmp_win;
         dragy += delta;
         dragHeight -= delta;
         action = 1;
+	cursor = TopCursor;
     }
     else if (y_root <= dragy/* ||
              y_root == findRootInfo(root)->rooty*/) {
@@ -413,6 +462,7 @@ TwmWindow *tmp_win;
         clampTop = 1;
 	clampDY = 0;
         action = 1;
+	cursor = TopCursor;
     }
     if (clampLeft) {
         int         delta = x_root - dragx;
@@ -423,6 +473,7 @@ TwmWindow *tmp_win;
         dragx += delta;
         dragWidth -= delta;
         action = 1;
+	cursor = clampTop ? TopLeftCursor : LeftCursor;
     }
     else if (x_root <= dragx/* ||
              x_root == findRootInfo(root)->rootx*/) {
@@ -433,6 +484,7 @@ TwmWindow *tmp_win;
         clampLeft = 1;
 	clampDX = 0;
         action = 1;
+	cursor = clampTop ? TopLeftCursor : LeftCursor;
     }
     if (clampBottom) {
         int         delta = y_root - dragy - dragHeight;
@@ -442,6 +494,7 @@ TwmWindow *tmp_win;
         }
         dragHeight += delta;
         action = 1;
+	cursor = clampLeft ? BottomLeftCursor : BottomCursor;
     }
     else if (y_root >= dragy + dragHeight - 1/* ||
            y_root == findRootInfo(root)->rooty
@@ -452,6 +505,7 @@ TwmWindow *tmp_win;
         clampBottom = 1;
 	clampDY = 0;
         action = 1;
+	cursor = clampLeft ? BottomLeftCursor : BottomCursor;
     }
     if (clampRight) {
         int         delta = x_root - dragx - dragWidth;
@@ -461,6 +515,8 @@ TwmWindow *tmp_win;
         }
         dragWidth += delta;
         action = 1;
+	cursor = clampBottom ? BottomRightCursor : RightCursor;
+	cursor = clampTop ? TopRightCursor : cursor;
     }
     else if (x_root >= dragx + dragWidth - 1/* ||
              x_root == findRootInfo(root)->rootx +
@@ -471,6 +527,8 @@ TwmWindow *tmp_win;
         clampRight = 1;
 	clampDX = 0;
         action = 1;
+	cursor = clampBottom ? BottomRightCursor : RightCursor;
+	cursor = clampTop ? TopRightCursor : cursor;
     }
 
     if (action) {
@@ -488,7 +546,11 @@ TwmWindow *tmp_win;
             dragy - tmp_win->frame_bw,
             dragWidth + 2 * tmp_win->frame_bw,
             dragHeight + 2 * tmp_win->frame_bw,
-	    tmp_win->frame_bw, tmp_win->title_height);
+	    tmp_win->frame_bw, tmp_win->title_height + tmp_win->frame_bw3D);
+	if (Scr->BorderCursors && (cursor != tmp_win->curcurs)) {
+	    tmp_win->curcurs = cursor;
+	    XChangeActivePointerGrab (dpy, resizeGrabMask, cursor, CurrentTime);
+	}
     }
 
     DisplaySize(tmp_win, dragWidth, dragHeight);
@@ -523,8 +585,8 @@ int height;
     last_width = width;
     last_height = height;
 
-    dheight = height - tmp_win->title_height;
-    dwidth = width;
+    dheight = height - tmp_win->title_height - 2 * tmp_win->frame_bw3D;
+    dwidth = width - 2 * tmp_win->frame_bw3D;
 
     /*
      * ICCCM says that PMinSize is the default is no PBaseSize is given,
@@ -592,14 +654,14 @@ EndResize()
 	int ncols = tmp_win->iconmgrp->cur_columns;
 	if (ncols == 0) ncols = 1;
 
-	tmp_win->iconmgrp->width = (int) ((dragWidth *
-					   (long) tmp_win->iconmgrp->columns)
+	tmp_win->iconmgrp->width = (int) (((dragWidth - 2 * tmp_win->frame_bw3D) *
+			(long) tmp_win->iconmgrp->columns)
 					  / ncols);
         PackIconManager(tmp_win->iconmgrp);
     }
 
     if (!Scr->NoRaiseResize)
-        XRaiseWindow(dpy, tmp_win->frame);
+        RaiseWindow(tmp_win);
 
     UninstallRootColormap();
 
@@ -669,7 +731,8 @@ ConstrainSize (tmp_win, widthp, heightp)
     int dwidth = *widthp, dheight = *heightp;
 
 
-    dheight -= tmp_win->title_height;
+    dwidth  -= 2 * tmp_win->frame_bw3D;
+    dheight -= (tmp_win->title_height + 2 * tmp_win->frame_bw3D);
 
     if (tmp_win->hints.flags & PMinSize) {
         minWidth = tmp_win->hints.min_width;
@@ -776,8 +839,8 @@ ConstrainSize (tmp_win, widthp, heightp)
     /*
      * Fourth, account for border width and title height
      */
-    *widthp = dwidth;
-    *heightp = dheight + tmp_win->title_height;
+    *widthp = dwidth + 2 * tmp_win->frame_bw3D;
+    *heightp = dheight + tmp_win->title_height + 2 * tmp_win->frame_bw3D;
 }
 
 
@@ -839,8 +902,8 @@ void SetupFrame (tmp_win, x, y, w, h, bw, sendEvent)
       bw = tmp_win->frame_bw;		/* -1 means current frame width */
 
     if (tmp_win->iconmgr) {
-	tmp_win->iconmgrp->width = w;
-        h = tmp_win->iconmgrp->height + tmp_win->title_height;
+	tmp_win->iconmgrp->width = w - (2 * tmp_win->frame_bw3D);
+        h = tmp_win->iconmgrp->height + tmp_win->title_height + (2 * tmp_win->frame_bw3D);
     }
 
     /*
@@ -854,9 +917,8 @@ void SetupFrame (tmp_win, x, y, w, h, bw, sendEvent)
       sendEvent = TRUE;
 
     xwcm = CWWidth;
-    title_width = xwc.width = w;
+    title_width  = xwc.width = w - (2 * tmp_win->frame_bw3D);
     title_height = Scr->TitleHeight + bw;
-
     ComputeWindowTitleOffsets (tmp_win, xwc.width, True);
 
     reShape = (tmp_win->wShaped ? TRUE : FALSE);
@@ -885,19 +947,16 @@ void SetupFrame (tmp_win, x, y, w, h, bw, sendEvent)
     if (tmp_win->title_w) {
 	if (bw != tmp_win->frame_bw) {
 	    xwc.border_width = bw;
-	    tmp_win->title_x = xwc.x = -bw;
-	    tmp_win->title_y = xwc.y = -bw;
+	    tmp_win->title_x = xwc.x = tmp_win->frame_bw3D - bw;
+	    tmp_win->title_y = xwc.y = tmp_win->frame_bw3D - bw;
 	    xwcm |= (CWX | CWY | CWBorderWidth);
 	}
 	
 	XConfigureWindow(dpy, tmp_win->title_w, xwcm, &xwc);
     }
 
-    tmp_win->attr.width = w;
-    tmp_win->attr.height = h - tmp_win->title_height;
-
-    XMoveResizeWindow (dpy, tmp_win->w, 0, tmp_win->title_height,
-		       w, h - tmp_win->title_height);
+    tmp_win->attr.width  = w - (2 * tmp_win->frame_bw3D);
+    tmp_win->attr.height = h - tmp_win->title_height - (2 * tmp_win->frame_bw3D);
 
     /* 
      * fix up frame and assign size/location values in tmp_win
@@ -914,42 +973,59 @@ void SetupFrame (tmp_win, x, y, w, h, bw, sendEvent)
     frame_mask |= (CWX | CWY | CWWidth | CWHeight);
     XConfigureWindow (dpy, tmp_win->frame, frame_mask, &frame_wc);
 
+    XMoveResizeWindow (dpy, tmp_win->w, tmp_win->frame_bw3D,
+			tmp_win->title_height + tmp_win->frame_bw3D,
+			tmp_win->attr.width, tmp_win->attr.height);
+
     /*
      * fix up highlight window
      */
-    if (tmp_win->title_height && tmp_win->hilite_w)
+    if (tmp_win->title_height && tmp_win->hilite_wl)
     {
-	xwc.width = (tmp_win->rightx - tmp_win->highlightx);
-	if (Scr->TBInfo.nright > 0) xwc.width -= Scr->TitlePadding;
-	if (Scr->use3Dtitles) xwc.width -= 4;
+	xwc.width = (tmp_win->name_x - tmp_win->highlightxl - 2);
         if (xwc.width <= 0) {
             xwc.x = Scr->MyDisplayWidth;	/* move offscreen */
             xwc.width = 1;
         } else {
-            xwc.x = tmp_win->highlightx;
+            xwc.x = tmp_win->highlightxl;
         }
 
         xwcm = CWX | CWWidth;
-        XConfigureWindow(dpy, tmp_win->hilite_w, xwcm, &xwc);
+        XConfigureWindow(dpy, tmp_win->hilite_wl, xwcm, &xwc);
+    }
+    if (tmp_win->title_height && tmp_win->hilite_wr)
+    {
+	xwc.width = (tmp_win->rightx - tmp_win->highlightxr);
+	if (Scr->TBInfo.nright > 0) xwc.width -= Scr->TitlePadding;
+	if (Scr->use3Dtitles) xwc.width -= Scr->TitleButtonShadowDepth;
+        if (xwc.width <= 0) {
+            xwc.x = Scr->MyDisplayWidth;	/* move offscreen */
+            xwc.width = 1;
+        } else {
+            xwc.x = tmp_win->highlightxr;
+        }
+
+        xwcm = CWX | CWWidth;
+        XConfigureWindow(dpy, tmp_win->hilite_wr, xwcm, &xwc);
     }
 
     if (HasShape && reShape) {
 	SetFrameShape (tmp_win);
     }
-    WMapSetupWindow (tmp_win, x, y + tmp_win->title_height, w, h - tmp_win->title_height);
-
+    WMapSetupWindow (tmp_win, x, y, w, h);
     if (sendEvent)
     {
         client_event.type = ConfigureNotify;
         client_event.xconfigure.display = dpy;
         client_event.xconfigure.event = tmp_win->w;
         client_event.xconfigure.window = tmp_win->w;
-        client_event.xconfigure.x = (x + tmp_win->frame_bw - tmp_win->old_bw);
+        client_event.xconfigure.x = (x + tmp_win->frame_bw - tmp_win->old_bw
+					+ tmp_win->frame_bw3D);
         client_event.xconfigure.y = (y + tmp_win->frame_bw +
-				     tmp_win->title_height - tmp_win->old_bw);
-        client_event.xconfigure.width = tmp_win->frame_width;
-        client_event.xconfigure.height = tmp_win->frame_height -
-                tmp_win->title_height;
+				     tmp_win->title_height - tmp_win->old_bw
+					+ tmp_win->frame_bw3D);
+        client_event.xconfigure.width = tmp_win->attr.width;
+        client_event.xconfigure.height = tmp_win->attr.height;
         client_event.xconfigure.border_width = tmp_win->old_bw;
         /* Real ConfigureNotify events say we're above title window, so ... */
 	/* what if we don't have a title ????? */
@@ -1062,12 +1138,14 @@ int flag;
       }
 
     if (!Scr->NoRaiseResize)
-        XRaiseWindow(dpy, tmp_win->frame);
+        RaiseWindow(tmp_win);
 
     ConstrainSize(tmp_win, &dragWidth, &dragHeight);
 
     SetupWindow (tmp_win, dragx , dragy , dragWidth, dragHeight, -1);
+/* I don't understand the reason of this. Claude.
     XUngrabPointer (dpy, CurrentTime);
+*/
     XUngrabServer (dpy);
 }
 
@@ -1094,7 +1172,7 @@ SetFrameShape (tmp)
 	 * need to do general case
 	 */
 	XShapeCombineShape (dpy, tmp->frame, ShapeBounding,
-			    0, tmp->title_height, tmp->w,
+			    tmp->frame_bw3D, tmp->title_height + tmp->frame_bw3D, tmp->w,
 			    ShapeBounding, ShapeSet);
 	if (tmp->title_w) {
 	    XShapeCombineShape (dpy, tmp->frame, ShapeBounding,
@@ -1120,25 +1198,25 @@ SetFrameShape (tmp)
 	     * The frame_width and frame_height do *not* include borders.
 	     */
 	    /* border */
-	    newBounding[0].x = tmp->title_x;
-	    newBounding[0].y = tmp->title_y;
-	    newBounding[0].width = tmp->title_width + fbw2;
+	    newBounding[0].x = tmp->title_x - tmp->frame_bw3D;
+	    newBounding[0].y = tmp->title_y - tmp->frame_bw3D;
+	    newBounding[0].width = tmp->title_width + fbw2 + 2 * tmp->frame_bw3D;
 	    newBounding[0].height = tmp->title_height;
 	    newBounding[1].x = -tmp->frame_bw;
 	    newBounding[1].y = Scr->TitleHeight;
-	    newBounding[1].width = tmp->attr.width + fbw2;
-	    newBounding[1].height = tmp->attr.height + fbw2;
+	    newBounding[1].width = tmp->attr.width + fbw2 + 2 * tmp->frame_bw3D;
+	    newBounding[1].height = tmp->attr.height + fbw2 + 2 * tmp->frame_bw3D;
 	    XShapeCombineRectangles (dpy, tmp->frame, ShapeBounding, 0, 0,
 				     newBounding, 2, ShapeSet, YXBanded);
 	    /* insides */
-	    newClip[0].x = tmp->title_x + tmp->frame_bw;
+	    newClip[0].x = tmp->title_x + tmp->frame_bw - tmp->frame_bw3D;
 	    newClip[0].y = 0;
-	    newClip[0].width = tmp->title_width;
-	    newClip[0].height = Scr->TitleHeight;
+	    newClip[0].width = tmp->title_width + 2 * tmp->frame_bw3D;
+	    newClip[0].height = Scr->TitleHeight + tmp->frame_bw3D;
 	    newClip[1].x = 0;
 	    newClip[1].y = tmp->title_height;
-	    newClip[1].width = tmp->attr.width;
-	    newClip[1].height = tmp->attr.height;
+	    newClip[1].width = tmp->attr.width + 2 * tmp->frame_bw3D;
+	    newClip[1].height = tmp->attr.height + 2 * tmp->frame_bw3D;
 	    XShapeCombineRectangles (dpy, tmp->frame, ShapeClip, 0, 0,
 				     newClip, 2, ShapeSet, YXBanded);
 	} else {
