@@ -24,6 +24,31 @@
 /**    TORTIOUS ACTION, ARISING OUT OF OR IN  CONNECTION  WITH  THE  USE    **/
 /**    OR PERFORMANCE OF THIS SOFTWARE.                                     **/
 /*****************************************************************************/
+/* 
+ *  [ ctwm ]
+ *
+ *  Copyright 1992 Claude Lecommandeur.
+ *            
+ * Permission to use, copy, modify  and distribute this software  [ctwm] and
+ * its documentation for any purpose is hereby granted without fee, provided
+ * that the above  copyright notice appear  in all copies and that both that
+ * copyright notice and this permission notice appear in supporting documen-
+ * tation, and that the name of  Claude Lecommandeur not be used in adverti-
+ * sing or  publicity  pertaining to  distribution of  the software  without
+ * specific, written prior permission. Claude Lecommandeur make no represen-
+ * tations  about the suitability  of this software  for any purpose.  It is
+ * provided "as is" without express or implied warranty.
+ *
+ * Claude Lecommandeur DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
+ * INCLUDING ALL  IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS.  IN NO
+ * EVENT SHALL  Claude Lecommandeur  BE LIABLE FOR ANY SPECIAL,  INDIRECT OR
+ * CONSEQUENTIAL  DAMAGES OR ANY  DAMAGES WHATSOEVER  RESULTING FROM LOSS OF
+ * USE, DATA  OR PROFITS,  WHETHER IN AN ACTION  OF CONTRACT,  NEGLIGENCE OR
+ * OTHER  TORTIOUS ACTION,  ARISING OUT OF OR IN  CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
+ *
+ * Author:  Claude Lecommandeur [ lecom@sic.epfl.ch ][ April 1992 ]
+ */
 
 
 /***********************************************************************
@@ -39,20 +64,27 @@
 #ifndef _TWM_
 #define _TWM_
 
+#ifdef VMS
+#include <decw$include/Xlib.h>
+#include <decw$include/Xutil.h>
+#include <decw$include/Intrinsic.h>
+#include <decw$include/cursorfont.h>
+#include <decw$include/shape.h>
+#ifndef X11R4
+#    include <decw$include/Xfuncs.h>
+#endif
+#else
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#include <X11/Intrinsic.h>
 #include <X11/cursorfont.h>
 #include <X11/extensions/shape.h>
 #ifndef X11R4
 #    include <X11/Xfuncs.h>
 #endif
-#include "list.h"
-
-#include <X11/Intrinsic.h>
-
-#if defined(XPM)
-#   include "util.h"
 #endif
+#include "list.h"
+#include "util.h"
 
 #ifndef WithdrawnState
 #define WithdrawnState 0
@@ -72,6 +104,10 @@
 
 typedef SIGNAL_T (*SigProc)();	/* type of function returned by signal() */
 
+#if defined(USE_SIGNALS) && defined(SVR4) && !defined(__sgi)
+#define signal sigset
+#endif /* SVR4 */
+
 #define BW 2			/* border width */
 #define BW2 4			/* border width  * 2 */
 
@@ -89,6 +125,12 @@ typedef SIGNAL_T (*SigProc)();	/* type of function returned by signal() */
 #define INFO_SIZE 200
 
 /* contexts for button presses */
+#define Alt1Mask 	(1<<8)
+#define Alt2Mask 	(1<<9)
+#define Alt3Mask 	(1<<10)
+#define Alt4Mask 	(1<<11)
+#define Alt5Mask 	(1<<12)
+
 #define C_NO_CONTEXT	-1
 #define C_WINDOW	0
 #define C_TITLE		1
@@ -98,7 +140,8 @@ typedef SIGNAL_T (*SigProc)();	/* type of function returned by signal() */
 #define C_ICONMGR	5
 #define C_NAME		6
 #define C_IDENTIFY      7
-#define NUM_CONTEXTS	8
+#define C_ALTERNATE     8
+#define NUM_CONTEXTS	9
 
 #define C_WINDOW_BIT	(1 << C_WINDOW)
 #define C_TITLE_BIT	(1 << C_TITLE)
@@ -107,6 +150,7 @@ typedef SIGNAL_T (*SigProc)();	/* type of function returned by signal() */
 #define C_FRAME_BIT	(1 << C_FRAME)
 #define C_ICONMGR_BIT	(1 << C_ICONMGR)
 #define C_NAME_BIT	(1 << C_NAME)
+#define C_ALTER_BIT	(1 << C_ALTERNATE)
 
 #define C_ALL_BITS	(C_WINDOW_BIT | C_TITLE_BIT | C_ICON_BIT |\
 			 C_ROOT_BIT | C_FRAME_BIT | C_ICONMGR_BIT)
@@ -148,23 +192,26 @@ typedef struct ColorPair
 
 typedef enum {on, off} ButtonState;
 
+typedef struct _TitleButtonFunc {
+    int func;                         /* function to execute */
+    char *action;                     /* optional action arg */
+    struct MenuRoot *menuroot;                /* menu to pop on F_MENU */
+} TitleButtonFunc;
+
 typedef struct _TitleButton {
     struct _TitleButton *next;		/* next link in chain */
     char *name;				/* bitmap name in case of deferal */
-    Pixmap bitmap;			/* image to display in button */
-    int depth;				/* depth of the image to display in button */
+    Image *image;			/* image to display in button */
     int srcx, srcy;			/* from where to start copying */
     unsigned int width, height;		/* size of pixmap */
     int dstx, dsty;			/* to where to start copying */
-    int func;				/* function to execute */
-    char *action;			/* optional action arg */
-    struct MenuRoot *menuroot;		/* menu to pop on F_MENU */
     Bool rightside;			/* t: on right, f: on left */
+    TitleButtonFunc funs[MAX_BUTTONS];  /* funcs assoc'd to each button */
 } TitleButton;
 
 typedef struct _TBWindow {
     Window window;			/* which window in this frame */
-    Pixmap bitmap;			/* image to display in button */
+    Image *image;			/* image to display in button */
     TitleButton *info;			/* description of this window */
 } TBWindow;
 
@@ -177,6 +224,9 @@ typedef struct _SqueezeInfo {
 #define J_LEFT			1
 #define J_CENTER		2
 #define J_RIGHT			3
+#define J_BORDER		4
+#define J_TOP			5
+#define J_BOTTOM		6
 
 /* Colormap window entry for each window in WM_COLORMAP_WINDOWS
  * ICCCM property.
@@ -219,18 +269,18 @@ typedef struct Icon
     Matchtype	match;
     Window	w;		/* the icon window */
     Window	bm_w;		/* the icon bitmap window */
-#if defined (XPM)
-    XpmIcon	*xpmicon;	/* xpm icon structure */
-#endif
+    Image	*image;		/* image icon structure */
     int		x;		/* icon text x coordinate */
     int		y;		/* icon text y coordiante */
     int		w_width;	/* width of the icon window */
     int		w_height;	/* height of the icon window */
-    int		width;		/* width of the icon bitmap */
-    int		height;		/* height of the icon bitmap */
+    int		width;          /* width of the icon bitmap */
+    int		height;         /* height of the icon bitmap */
     char	*pattern;	/* Why this icon was choosed */
     Pixel	border;		/* border color */
     ColorPair	iconc;
+    Bool	has_title;
+    int		border_width;
 } Icon;
 
 /* for each window that is on the display, one of these structures
@@ -244,7 +294,9 @@ typedef struct TwmWindow
     int old_bw;			/* border width before reparenting */
     Window frame;		/* the frame window */
     Window title_w;		/* the title bar window */
-    Window hilite_w;		/* the hilite window */
+    Window hilite_wl;		/* the left hilite window */
+    Window hilite_wr;		/* the right hilite window */
+    Cursor curcurs;		/* current resize cursor */
     Pixmap gray;
     Icon *icon;			/* the curent icon */
     name_list *iconslist;	/* the current list of icons */
@@ -253,6 +305,7 @@ typedef struct TwmWindow
     int frame_width;		/* width of frame */
     int frame_height;		/* height of frame */
     int frame_bw;		/* borderwidth of frame */
+    int frame_bw3D;		/* 3D borderwidth of frame */
     int title_x;
     int title_y;
     int title_height;		/* height of the title bar */
@@ -260,8 +313,10 @@ typedef struct TwmWindow
     char *full_name;		/* full name of the window */
     char *name;			/* name of the window */
     char *icon_name;            /* name of the icon */
+    int name_x;			/* start x of name text */
     int name_width;		/* width of name text */
-    int highlightx;		/* start of highlight window */
+    int highlightxl;		/* start of left highlight window */
+    int highlightxr;		/* start of right highlight window */
     int rightx;			/* start of right buttons */
     XWindowAttributes attr;	/* the child window attributes */
     XSizeHints hints;		/* normal hints */
@@ -272,7 +327,7 @@ typedef struct TwmWindow
     /***********************************************************************
      * color definitions per window
      **********************************************************************/
-    Pixel border;		/* border color */
+    ColorPair borderC;		/* border color */
     ColorPair border_tile;
     ColorPair title;
     short iconified;		/* has the window ever been iconified? */
@@ -285,6 +340,7 @@ typedef struct TwmWindow
     short icon_moved;		/* user explicitly moved the icon */
     short highlight;		/* should highlight this window */
     short stackmode;		/* honor stackmode requests */
+    short ontoppriority;	/* how much on top should that be */
     short iconify_by_unmapping;	/* unmap window to iconify it */
     short iconmgr;		/* this is an icon manager window */
     short transient;		/* this is a transient window */
@@ -309,8 +365,9 @@ typedef struct TwmWindow
 
     short OpaqueMove;
     short OpaqueResize;
+    Bool hasfocusvisible;	/* The window has visivle focus*/
     int  occupation;
-
+    Image *HiliteImage;                /* focus highlight window background */
 } TwmWindow;
 
 #define DoesWmTakeFocus		(1L << 0)
@@ -332,8 +389,12 @@ typedef struct TwmWindow
 #define TBPM_3DBAR ":xpm:bar"
 
 #ifndef X11R4
+#ifdef VMS
+#    include <decw$include/Xosdefs.h>
+#else
 #    include <X11/Xosdefs.h>
 #endif
+#endif
 #ifndef X_NOT_STDC_ENV
 #include <stdlib.h>
 #else
@@ -380,7 +441,9 @@ extern int InfoLines;
 extern char Info[][INFO_SIZE];
 extern int Argc;
 extern char **Argv;
+#ifndef VMS
 extern char **Environ;
+#endif
 extern void NewFontCursor();
 extern Pixmap CreateMenuIcon();
 extern Pixmap Create3DMenuIcon();
@@ -403,5 +466,6 @@ extern Atom _XA_WM_SAVE_YOURSELF;
 extern Atom _XA_WM_DELETE_WINDOW;
 
 #define OCCUPY(w, b) ((b == NULL) ? 1 : (w->occupation & (1 << b->number)))
+#define VISIBLE(w) (w->occupation & Scr->workSpaceMgr.visibility)
 
 #endif /* _TWM_ */
